/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * Portions Copyrighted 2010 Jaroslav Tulach
 */

package cz.xelfi.quoridor.freemarkerdor;

import com.sun.jersey.api.client.Client;
import com.sun.jersey.api.client.ClientResponse;
import com.sun.jersey.api.client.WebResource;
import com.sun.jersey.core.header.MediaTypes;
import com.sun.jersey.core.util.MultivaluedMapImpl;
import com.sun.net.httpserver.HttpServer;
import cz.xelfi.quoridor.webidor.resources.Quoridor;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.URI;
import java.util.Locale;
import java.util.concurrent.Callable;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import javax.ws.rs.core.Cookie;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.MultivaluedMap;
import javax.ws.rs.core.UriBuilder;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;
import org.netbeans.junit.MockServices;
import static org.junit.Assert.*;

/**
 *
 * @author Jaroslav Tulach <jtulach@netbeans.org>
 */
public class ChangeEmailTest extends Object {
    private static File dir;
    private static HttpServer stopAPI;
    private static HttpServer stopStatistics;
    private static Callable<Void> stop;
    private WebResource webResource;
    private WebResource apiResource;
    private WebResource statResource;
    private Client client;

    public ChangeEmailTest() throws Exception {
    }

    @BeforeClass
    public static void localeEnglish() throws Exception {
        Locale.setDefault(Locale.ENGLISH);
        dir = File.createTempFile("quoridor", ".dir");
        dir.delete();
        dir.mkdirs();
        System.setProperty("quoridor.dir", dir.getPath());
        stopAPI = Quoridor.start(7990);
        stopStatistics = Quoridor.start(7992);
        stop = UI.startServers(7991, "http://localhost:7990", "http://localhost:7992", null);

        File passwd = new File(dir, "passwd");
        FileOutputStream os = new FileOutputStream(passwd);
        os.write("test=pes\nJarda=darda\n".getBytes("UTF-8"));
        os.close();

        MockServices.setServices(MockEmailer.class);
    }

    @Before
    public void setUp() throws Exception {
        MockEmailer.clear();

        client = new Client();
        webResource = client.resource(new URI("http://localhost:7991/"));
        apiResource = client.resource(new URI("http://localhost:7990/"));
        statResource = client.resource(new URI("http://localhost:7992/"));
    }

    @AfterClass
    public static void cleanUpAll() throws Exception {
        deleteRec(dir);
        if (stop != null) {
            stop.call();
        }
        if (stopAPI != null) {
            stopAPI.stop(0);
        }
        MockServices.setServices();
    }

    static void deleteRec(File dir) throws IOException {
        if (dir == null) {
            return;
        }
        File[] arr = dir.listFiles();
        if (arr != null) {
            for (File f : arr) {
                deleteRec(f);
            }
        }
        dir.delete();
    }

    @Test public void testChangeEmail() throws Exception {
        String logTest = apiResource.path("login").
            queryParam("name", "test").
            queryParam("password", "pes").
            accept(MediaType.TEXT_PLAIN).
            put(String.class);
        ClientResponse res = webResource.path("options").
            queryParam("email", "xxx@xxx.cz").
            cookie(Cookie.valueOf("login=" + logTest)).
            accept("text/html").
            get(ClientResponse.class);
        final String txt = res.getEntity(String.class);

        assertEquals("Emails sent to", "xxx@xxx.cz", MockEmailer.address);

        int urlIndex = MockEmailer.text.indexOf(webResource.getURI().toString());
        assertTrue("Found in " + MockEmailer.text, urlIndex >= 0);

        int endIndex = urlIndex;
        while (!Character.isWhitespace(MockEmailer.text.charAt(endIndex))) {
            endIndex++;
        }
        final String url = MockEmailer.text.substring(urlIndex, endIndex);
        WebResource requestURL = client.resource(url);

        String response = requestURL.accept("text/html").get(String.class);
        Pattern p = Pattern.compile("<input *type=\"text\" *name=\"email\" *value='xxx@xxx.cz'");
        assertTrue("New email found" + response, p.matcher(response).find());
    }

    public static final class MockEmailer extends EmailService {
        static String address;
        static String subject;
        static String text;

        @Override
        public void sendEmail(String address, String subject, String text) throws IOException {
            assertNull("No email sent yet", MockEmailer.address);
            MockEmailer.address = address;
            MockEmailer.subject = subject;
            MockEmailer.text = text;
        }

        public static void clear() {
            MockEmailer.address = null;
            MockEmailer.subject = null;
            MockEmailer.text = null;
        }
    }
}
